import { Resolver, Query, Mutation, Args, Int } from '@nestjs/graphql';
import { DiscussionMasterTableService } from './discussion-master-table.service';
import { DiscussionMasterTable } from './entities/discussion-master-table.entity';
import { CreateDiscussionMasterTableInput } from './dto/create-discussion-master-table.input';
import { UpdateDiscussionMasterTableInput } from './dto/update-discussion-master-table.input';
import { Roles } from 'src/user-collection/decorators/roles.decorator';
import { Role } from 'src/user-collection/roles/role.enum';
import { GqlAuthGuard } from 'src/auth/guards/gql-auth.guard';
import { RolesGuard } from 'src/auth/guards/roles.guard';
import { UseGuards } from '@nestjs/common';
import { DiscussionMasterTableReturnType } from './types/discussion-master-table-return.type';
import { DiscussionMasterTableType } from './types/discussion-mater-table.type';
import { CurrentUser } from 'src/user-collection/decorators/user.decorator';

@Resolver(() => DiscussionMasterTable)
export class DiscussionMasterTableResolver {
  constructor(private readonly discussionMasterTableService: DiscussionMasterTableService) {}

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => DiscussionMasterTableReturnType)
  createDiscussionMasterTable(@Args('createDiscussionMasterTableInput') createDiscussionMasterTableInput: CreateDiscussionMasterTableInput, @CurrentUser() user: any) {
    return this.discussionMasterTableService.create(createDiscussionMasterTableInput, user);
  }

  

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [DiscussionMasterTableType])
  findAllDiscussionMasterByCourseId(@Args('courseId', { type: () => Int }) courseId: number) {
    return this.discussionMasterTableService.findAll(courseId);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => DiscussionMasterTableType)
  findOneDiscussionMaster(@Args('discussionMasterId', { type: () => Int }) discussionMasterId: number) {
    return this.discussionMasterTableService.findOne(discussionMasterId);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => DiscussionMasterTableReturnType)
  updateDiscussionMasterTable(@Args('updateDiscussionMasterTableInput') updateDiscussionMasterTableInput: UpdateDiscussionMasterTableInput, @CurrentUser() user: any) {
    return this.discussionMasterTableService.update(updateDiscussionMasterTableInput.discussionMasterId, updateDiscussionMasterTableInput, user);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => DiscussionMasterTableReturnType)
  removeDiscussionMasterTable(@Args('discussionMasterId', { type: () => Int }) discussionMasterId: number) {
    return this.discussionMasterTableService.remove(discussionMasterId);
  }
}
