import { Inject, Injectable, forwardRef } from '@nestjs/common';
import { CreateDiscussionMasterTableInput } from './dto/create-discussion-master-table.input';
import { UpdateDiscussionMasterTableInput } from './dto/update-discussion-master-table.input';
import { DiscussionMasterTable } from './entities/discussion-master-table.entity';
import { Repository } from 'typeorm';
import { InjectRepository } from '@nestjs/typeorm';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { CourseTableService } from 'src/course-table/course-table.service';
import { UserCollectionService } from 'src/user-collection/user-collection.service';

@Injectable()
export class DiscussionMasterTableService {

  constructor(
    @InjectRepository(DiscussionMasterTable)
    private readonly discussionMasterTableRepository: Repository<DiscussionMasterTable>,

    @Inject(PUB_SUB) private pubSub: RedisPubSub,

    @Inject(forwardRef(() => CourseTableService))
    private readonly courseTableService: CourseTableService,

    @Inject(forwardRef(() => UserCollectionService))
    private readonly userCollectionService: UserCollectionService,

  ) { }


  async create(createDiscussionMasterTableInput: CreateDiscussionMasterTableInput, user: any) {
    const checkIfExists = await this.checkIfExists(createDiscussionMasterTableInput, user);

    if(!checkIfExists){
      let dmt = new DiscussionMasterTable();

      dmt.CourseTable = await this.courseTableService.findOne(createDiscussionMasterTableInput.courseId);
      dmt.PostedBy = await this.userCollectionService.getOneByUserId(user.id);
      dmt.discussionTopic = createDiscussionMasterTableInput.discussionTopic;
      dmt.hasNewMessage=1;

      let result = await this.discussionMasterTableRepository.save(dmt);

      return {
        status: true,
        status_text: "Successfully Created",
        data: result
      }

    }else{
      return {
        status: false,
        status_text: "Topic already exists"
      }
    }
  }


  async checkIfExists(createDiscussionMasterTableInput: CreateDiscussionMasterTableInput, user: any){
    return await this.discussionMasterTableRepository.findOne({
      where: {
        CourseTable: {
          courseId: createDiscussionMasterTableInput.courseId
        },
        PostedBy: {
          id: user.id
        },
        discussionTopic: createDiscussionMasterTableInput.discussionTopic
      }
    })
  }

  async findAll(courseId: number) {
    return await this.discussionMasterTableRepository.find({
      where: {
        CourseTable: {
          courseId
        }
      },
      relations: {
        CourseTable: true,
        DiscussionTable: true,
        PostedBy: true
      },
      order: {
        discussionMasterId: "DESC"
      }
    })
  }

  async findOne(discussionMasterId: number) {
    return await this.discussionMasterTableRepository.findOne({
      where: {
        discussionMasterId
      },
      relations: {
        CourseTable: true,
        DiscussionTable: true,
        PostedBy: true
      }
    })
  }

  async update(discussionMasterId: number, updateDiscussionMasterTableInput: UpdateDiscussionMasterTableInput, user: any) {
    let result = await this.discussionMasterTableRepository.findOne({
      where: {
        discussionMasterId
      }
    })

    if(result){
      let dmt = result;
      dmt.CourseTable = await this.courseTableService.findOne(updateDiscussionMasterTableInput.courseId);
      dmt.PostedBy = await this.userCollectionService.getOneByUserId(user.id);
      dmt.discussionTopic = updateDiscussionMasterTableInput.discussionTopic;

      await this.discussionMasterTableRepository.save(dmt);

      return {
        status: true,
        status_text: "Successfully Updated"
      }
    }else{
      return {
        status: false,
        status_text: "Not Found"
      }
    }
  }

  async remove(discussionMasterId: number) {
    let result = await this.discussionMasterTableRepository.findOne({
      where: {
        discussionMasterId
      }
    })

    if(result){
      await this.discussionMasterTableRepository.remove(result);

      return {
        status: true,
        status_text: "Successfully Deleted"
      }
    }else{
      return {
        status: false,
        status_text: "Not Found"
      }
    }
  }


  async setHasNewMessage(discussionMasterId: number, value: number) {
    let result = await this.discussionMasterTableRepository.findOne({
      where: {
        discussionMasterId
      }
    })

    if(result){
      result.hasNewMessage = value;
      await this.discussionMasterTableRepository.save(result);
      return true;
    }else{
      return false;
    }
  }
}
