import { Resolver, Query, Mutation, Args, Int } from '@nestjs/graphql';
import { DiscussionTableService } from './discussion-table.service';
import { DiscussionTable } from './entities/discussion-table.entity';
import { CreateDiscussionTableInput } from './dto/create-discussion-table.input';
import { UpdateDiscussionTableInput } from './dto/update-discussion-table.input';
import { CurrentUser } from 'src/user-collection/decorators/user.decorator';
import { Roles } from 'src/user-collection/decorators/roles.decorator';
import { Role } from 'src/user-collection/roles/role.enum';
import { UseGuards } from '@nestjs/common';
import { GqlAuthGuard } from 'src/auth/guards/gql-auth.guard';
import { RolesGuard } from 'src/auth/guards/roles.guard';
import { DiscussionTableReturnType } from './types/discussion-table-return.type';
import { DiscussionTableType } from './types/discussion-table.type';

@Resolver(() => DiscussionTable)
export class DiscussionTableResolver {
  constructor(private readonly discussionTableService: DiscussionTableService) {}

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => DiscussionTableReturnType)
  createDiscussionTable(@Args('createDiscussionTableInput') createDiscussionTableInput: CreateDiscussionTableInput, @CurrentUser() user: any) {
    return this.discussionTableService.create(createDiscussionTableInput, user);
  }

  // @Query(() => [DiscussionTable], { name: 'discussionTable' })
  // findAll() {
  //   return this.discussionTableService.findAll();
  // }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [DiscussionTableType])
  findAllByCourseId(@Args('courseId', { type: () => Int }) courseId: number, @Args('discussionMasterId', { type: () => Int }) discussionMasterId: number) {
    return this.discussionTableService.findAllByCourseId(courseId, discussionMasterId);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [DiscussionTableType])
  findAllByCourseIdWeb(@Args('courseId', { type: () => Int }) courseId: number) {
    return this.discussionTableService.findAllByCourseIdWeb(courseId);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [DiscussionTableType])
  findAllByCourseIdWebDiscussionId(@Args('discussionMasterId', { type: () => Int }) discussionMasterId: number) {
    return this.discussionTableService.findAllByCourseIdWebDiscussionId(discussionMasterId);
  }

  // @Mutation(() => DiscussionTable)
  // updateDiscussionTable(@Args('updateDiscussionTableInput') updateDiscussionTableInput: UpdateDiscussionTableInput) {
  //   return this.discussionTableService.update(updateDiscussionTableInput.id, updateDiscussionTableInput);
  // }

  // @Mutation(() => DiscussionTable)
  // removeDiscussionTable(@Args('id', { type: () => Int }) id: number) {
  //   return this.discussionTableService.remove(id);
  // }
}
