import { Inject, Injectable, forwardRef } from '@nestjs/common';
import { CreateDiscussionTableInput } from './dto/create-discussion-table.input';
import { UpdateDiscussionTableInput } from './dto/update-discussion-table.input';
import { InjectRepository } from '@nestjs/typeorm';
import { DiscussionTable } from './entities/discussion-table.entity';
import { Repository } from 'typeorm';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { CourseTableService } from 'src/course-table/course-table.service';
import { UserCollectionService } from 'src/user-collection/user-collection.service';
import { DiscussionMasterTableService } from 'src/discussion-master-table/discussion-master-table.service';

@Injectable()
export class DiscussionTableService {
  constructor(
    @InjectRepository(DiscussionTable)
    private readonly discussionTableRepository: Repository<DiscussionTable>,

    @Inject(PUB_SUB) private pubSub: RedisPubSub,

    @Inject(forwardRef(() => DiscussionMasterTableService))
    private readonly discussionMasterTableService: DiscussionMasterTableService,

    @Inject(forwardRef(() => UserCollectionService))
    private readonly userCollectionService: UserCollectionService,

  ) { }

  async create(createDiscussionTableInput: CreateDiscussionTableInput, user: any) {


    let st = new DiscussionTable();

    let discussionMasterTable = await this.discussionMasterTableService.findOne(createDiscussionTableInput.discussionMasterId)
    st.DiscussionMasterTable = discussionMasterTable;


    let userTable = await this.userCollectionService.findOne(user.id)
    st.UserTable = userTable;

    st.message = createDiscussionTableInput.message;


    let result = await this.discussionTableRepository.save(st);

    if(userTable.role == "USER" || userTable.role == "STUDENT"){
      await this.discussionMasterTableService.setHasNewMessage(discussionMasterTable.discussionMasterId, 1);
    }


    // send to subscriber
    //this.pubSub.publish(NEW_COURSE_UNIT_ADD, 1);


    return {
      status: true,
      status_text: "Successfully Added",
      data: result
    }


  }



  async findAllByCourseId(courseId: number, discussionMasterId: number) {
    let result = await this.discussionTableRepository.find({
      where: {
        DiscussionMasterTable: {
          discussionMasterId
        }
      },
      relations: {
        UserTable: true
      },
      order: {
        discussionId: "DESC"
      }
    });
   

    let data = [];


    for(const el of result){
      let d = {
        _id: el.discussionId,
        text: el.message,
        createdAt: el.createdAt,
        user: {
          _id: el.UserTable.id,
          name: el.UserTable.name
        }
      }

      data.push(d)
    }
    console.log("🚀 ~ DiscussionTableService ~ findAllByCourseId ~ data:", data)
    return data;
  }
    

  async findAllByCourseIdWeb(courseId: number) {
    let result = await this.discussionTableRepository.find({
      where: {
        // CourseTable: {
        //   courseId
        // },
      },
      relations: {
        UserTable: true
      },
      order: {
        discussionId: "ASC"
      }
    });
   

    let data = [];


    for(const el of result){
      let d = {
        _id: el.discussionId,
        text: el.message,
        createdAt: el.createdAt,
        user: {
          _id: el.UserTable.id,
          name: el.UserTable.name
        }
      }

      data.push(d)
    }
    console.log("🚀 ~ DiscussionTableService ~ findAllByCourseId ~ data:", data)
    return data;
  }

  async findAllByCourseIdWebDiscussionId(discussionMasterId: number) {
    let result = await this.discussionTableRepository.find({
      where: {
        DiscussionMasterTable: {
          discussionMasterId
        }
      },
      relations: {
        UserTable: true
      },
      order: {
        discussionId: "ASC"
      }
    });
   

    let data = [];


    for(const el of result){
      let d = {
        _id: el.discussionId,
        text: el.message,
        createdAt: el.createdAt,
        user: {
          _id: el.UserTable.id,
          name: el.UserTable.name
        }
      }

      data.push(d)
    }
    
    // mark as seen
    this.discussionMasterTableService.setHasNewMessage(discussionMasterId, 0)

    return data;
  }
 

  // async update(courseUnitId: number, updateCourseUnitTableInput: UpdateCourseUnitTableInput) {
  //   let result = await this.findOne(courseUnitId);

  //   if (result) {

  //     let courseTable = await this.courseTableService.findOne(updateCourseUnitTableInput.courseId)
  //     result.CourseTable = courseTable;

  //     result.courseUnitName = updateCourseUnitTableInput.courseUnitName;
  //     result.courseUnitDescription = updateCourseUnitTableInput.courseUnitDescription;
  //     result.courseUnitDuration = updateCourseUnitTableInput.courseUnitDuration;
  //     result.numberOfPages = updateCourseUnitTableInput.numberOfPages;
  //     result.weightOfPercentage = updateCourseUnitTableInput.weightOfPercentage;
  //     result.courseFile = updateCourseUnitTableInput.courseFile;
  //     result.courseUnitBanner = updateCourseUnitTableInput.courseUnitBanner;


  //     result.hasAssesment = updateCourseUnitTableInput.hasAssesment;
  //     result.totalScore = updateCourseUnitTableInput.totalScore;
  //     result.passingScore = updateCourseUnitTableInput.passingScore;
  //     result.isLink = updateCourseUnitTableInput.isLink;
  //     result.link = updateCourseUnitTableInput.link;

  //     let updatedResult = await this.courseUnitTableRepository.save(result);


  //     // send to subscriber
  //     this.pubSub.publish(NEW_COURSE_UNIT_ADD, 1);


  //     return {
  //       status: true,
  //       status_text: "Successfully Updated",
  //       data: updatedResult
  //     }
  //   } else {
  //     return {
  //       status: false,
  //       status_text: "Not Found"
  //     }
  //   }
  // }

  // async remove(courseUnitId: number) {
  //   let result = await this.findOne(courseUnitId)

  //   if (result) {
  //     await this.courseUnitTableRepository.remove(result);


  //     // send to subscriber
  //     this.pubSub.publish(NEW_COURSE_UNIT_ADD, 1);


  //     return {
  //       status: true,
  //       status_text: "Successfully Deleted"
  //     }
  //   } else {
  //     return {
  //       status: false,
  //       status_text: "Not Found"
  //     }
  //   }
  // }


  // async toggleHasAssesmentofCourseUnit(courseUnitId: number) {
  //   let res = await this.courseUnitTableRepository.findOne({
  //     where: {
  //       courseUnitId
  //     }
  //   })


  //   if (res) {
  //     // update
  //     let oldStatus = res.hasAssesment;

  //     let newStatus = "YES";
  //     if (oldStatus == "YES") {
  //       newStatus = "NO";
  //     }

  //     if (oldStatus == "NO") {
  //       newStatus = "YES";
  //     }

  //     res.hasAssesment = newStatus;

  //     let result = await this.courseUnitTableRepository.save(res);


  //     return {
  //       status: true,
  //       status_text: "Successfully Updated",
  //       data: result
  //     }

  //   } else {
  //     // return error
  //     return {
  //       status: false,
  //       status_text: "Not found"
  //     }
  //   }
  // }
}
